@@ -17,10 +17,13 @@ if __name__ == '__main__':
 		logging.basicConfig(level = logging.INFO)
 
 	while resolution not in [1, 2, 4]:
-		resolution = int(input('Pulses per degree: '))
+		resolution = input('Pulses per degree: ')
+		try:
+			resolution = int(resolution)
+		except ValueError:
+			pass
 
 	port = input('Please enter the serial port: ')
-
 	sim = rot2prog.ROT2ProgSim(port, resolution)
 
 	logging.getLogger().info('Press [Enter] to close simulator')
